const path = require('path')
const os = require('os')
const fs = require('fs')
const cloneDeep = require('lodash.clonedeep')
const execa = require('execa')
const diff = require('deep-diff').diff
let changedDep = []
const installDep = async (context, dep, api) => {
  const { name, value, p } = dep
  dep = `${name}@${value}`
  const child = await execa('npm', ['install', dep], {
    cwd: context
  })
  const pkg = api.generator.pkg
  pkg[p][name] = value
  fs.writeFileSync(path.join(context, 'package.json'), JSON.stringify(pkg, null, 2))
}
const toAbsoluteVersion = async (prev, cur, p, api) => {
  const result = diff(prev[p], cur[p], api)
  try {
    result.forEach(v => {
      if (v.kind === 'E' && /^[^\^~]+$/.test(v.rhs)) {
        const name = v.path[0]
        const value = v.rhs
        changedDep.push({
          p,
          name,
          value
        })
      }
    })
  } catch (e) { }
}
/**
 * @param {Object} api 
 * @param {Object} options vue project config
 * @param {String} [options.basePath] base root
 * @param {String} [options.pkg] package.json except vue preset
 * @param {String} [options.templates] render templates from local
 */
module.exports = (api, options) => {
  const previousPkg = cloneDeep(api.generator.pkg)
  let { basePath, pkg, templates } = options
  basePath = api.resolve(basePath)

  //render
  api.render('./template')
  if (fs.existsSync(templates)) {
    api.render(templates)
  }

  if (pkg) {
    pkg = path.join(basePath, pkg)
    try {
      pkg = require(pkg)
      api.extendPackage(Object.assign({}, pkg))
      toAbsoluteVersion(previousPkg, pkg, 'dependencies', api)
    } catch (e) { }
  }
  api.onCreateComplete(async () => {
    //change to absolute version ^1.2.3=>1.2.3
    if (changedDep.length) {
      for (const dep of changedDep.slice()) {
        await installDep(api.resolve('./'), dep, api)
      }
      changedDep = []
    }

  })
}
